import React, { useState } from 'react';
import FormControl from '@material-ui/core/FormControl';
import { useApi, identityApiRef, discoveryApiRef, fetchApiRef } from '@backstage/core-plugin-api';
import { ErrorPanel, Progress, Select } from '@backstage/core-components';
import { useAsync } from 'react-use';
import get from 'lodash/get';
import sortBy from 'lodash/sortBy';
import { selectFieldFromApiConfigSchema } from '../../types.esm.js';
import { FormHelperText, Box, Typography, Button } from '@material-ui/core';
import { useOauthSignIn } from '../../hooks/useOauthSignIn.esm.js';
import { renderString } from 'nunjucks';
import fromPairs from 'lodash/fromPairs';
import isObject from 'lodash/isObject';
import toPairs from 'lodash/toPairs';

const renderOption = (input, context) => {
  if (!input) {
    return input;
  }
  if (typeof input === "string") {
    return renderString(input, context);
  }
  if (Array.isArray(input)) {
    return input.map((item) => renderOption(item, context));
  }
  if (typeof input === "object") {
    return fromPairs(
      Object.keys(input).map((key) => [
        key,
        renderOption(input[key], context)
      ])
    );
  }
  return input;
};
const SelectFieldFromApiComponent = (props) => {
  const discoveryApi = useApi(discoveryApiRef);
  const fetchApi = useApi(fetchApiRef);
  const [dropDownData, setDropDownData] = useState();
  const { formContext, uiSchema, identity, formData } = props;
  const isArrayField = props?.schema?.type === "array";
  const optionsParsingState = selectFieldFromApiConfigSchema.safeParse(
    uiSchema["ui:options"]
  );
  const { error } = useAsync(async () => {
    if (!optionsParsingState.success) {
      throw optionsParsingState.error;
    }
    const options = optionsParsingState.data;
    const baseUrl = await discoveryApi.getBaseUrl("");
    const headers = {};
    if (props.token) {
      headers.Authorization = `Bearer ${props.token}`;
    }
    let init = renderOption(options.params, {
      parameters: formContext.formData,
      identity
    });
    if (Array.isArray(init)) {
      init = init.reduce((acc, val) => {
        if (isObject(val)) {
          acc.push(toPairs(val).flat());
        } else {
          acc.push(val);
        }
        return acc;
      }, []);
    }
    const params = new URLSearchParams(init);
    const response = await fetchApi.fetch(
      `${baseUrl}${renderOption(options.path, {
        parameters: formContext.formData
      })}?${params}`,
      { headers }
    );
    const body = await response.json();
    const array = options.arraySelector ? get(
      body,
      renderOption(options.arraySelector, {
        parameters: formContext.formData
      })
    ) : body;
    const constructedData = array.map((item) => {
      let value;
      let label;
      if (options.valueSelector) {
        value = get(
          item,
          renderOption(options.valueSelector, {
            parameters: formContext.formData
          })
        );
        label = options.labelSelector ? get(
          item,
          renderOption(options.labelSelector, {
            parameters: formContext.formData
          })
        ) : value;
      } else {
        if (!(typeof item === "string")) {
          throw new Error(
            `The item provided for the select drop down "${item}" is not a string`
          );
        }
        value = item;
        label = item;
      }
      if (!value) {
        throw new Error(`Failed to populate SelectFieldFromApi dropdown`);
      }
      return {
        value,
        label: label || value
      };
    });
    setDropDownData(sortBy(constructedData, "label"));
  });
  const {
    title = "Select",
    description = "",
    placeholder = "Select from results"
  } = optionsParsingState.success ? optionsParsingState.data : {};
  if (error) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error });
  }
  if (!dropDownData) {
    return /* @__PURE__ */ React.createElement(Progress, null);
  }
  return /* @__PURE__ */ React.createElement(
    FormControl,
    {
      margin: "normal",
      required: props.required,
      error: (props?.rawErrors?.length || 0) > 0 && !formData
    },
    /* @__PURE__ */ React.createElement(
      Select,
      {
        items: dropDownData,
        selected: formData,
        placeholder,
        label: title,
        multiple: isArrayField,
        onChange: (selected) => {
          props.onChange(
            Array.isArray(selected) && !isArrayField ? void 0 : selected
          );
        }
      }
    ),
    /* @__PURE__ */ React.createElement(FormHelperText, null, description)
  );
};
const SelectFieldFromApiOauthWrapper = ({
  oauthConfig,
  ...props
}) => {
  const { token, loading, error, isSignedIn, showSignInModal } = useOauthSignIn(oauthConfig);
  if (loading && !isSignedIn) {
    return /* @__PURE__ */ React.createElement(Progress, null);
  }
  if (error) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error });
  }
  if (!props.uiSchema) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error: new Error("No UI Schema defined") });
  }
  if (!isSignedIn || !token) {
    return /* @__PURE__ */ React.createElement(Box, { height: "100%", width: "100%" }, /* @__PURE__ */ React.createElement(Box, null, /* @__PURE__ */ React.createElement(Typography, { variant: "body2" }, /* @__PURE__ */ React.createElement("b", null, props.uiSchema["ui:options"]?.title || props.name))), /* @__PURE__ */ React.createElement(Box, { display: "flex" }, /* @__PURE__ */ React.createElement(Box, { paddingRight: 1 }, /* @__PURE__ */ React.createElement(Typography, null, "This input requires authentication with ", oauthConfig.provider)), /* @__PURE__ */ React.createElement(Box, null, /* @__PURE__ */ React.createElement(
      Button,
      {
        variant: "outlined",
        color: "primary",
        onClick: showSignInModal,
        size: "small"
      },
      "Sign In"
    ))));
  }
  return /* @__PURE__ */ React.createElement(
    SelectFieldFromApiComponent,
    {
      ...{ ...props, uiSchema: props.uiSchema },
      token
    }
  );
};
const SelectFieldFromApi = (props) => {
  const identityApi = useApi(identityApiRef);
  const { loading, value: identity } = useAsync(async () => {
    return await identityApi.getBackstageIdentity();
  });
  if (!props.uiSchema) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error: new Error("No UI Schema defined") });
  }
  const result = selectFieldFromApiConfigSchema.safeParse(
    props.uiSchema["ui:options"]
  );
  if (loading) {
    return /* @__PURE__ */ React.createElement(React.Fragment, null);
  }
  if (result.success && result.data.oauth) {
    return /* @__PURE__ */ React.createElement(
      SelectFieldFromApiOauthWrapper,
      {
        identity,
        oauthConfig: result.data.oauth,
        ...props
      }
    );
  }
  return /* @__PURE__ */ React.createElement(
    SelectFieldFromApiComponent,
    {
      ...{ ...props, uiSchema: props.uiSchema },
      identity
    }
  );
};

export { SelectFieldFromApi };
//# sourceMappingURL=SelectFieldFromApi.esm.js.map
